// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef XPLRA_MULTITASKREQUEST_H
#define XPLRA_MULTITASKREQUEST_H
//------------------------------------------------------------------------------

#include "Request.h"

#include <vector>

#include <cstdlib>

//------------------------------------------------------------------------------

namespace xplra {

//------------------------------------------------------------------------------

class Task;

//------------------------------------------------------------------------------

/**
 * A request containing several tasks which it performs one-by-one
 * when executing the request. It owns the tasks, which get deleted
 * when the request object is deleted.
 */
class MultiTaskRequest : public Request
{
private:
    /**
     * Type for the vector of the tasks.
     */
    typedef std::vector<Task*> tasks_t;

    /**
     * The tasks.
     */
    tasks_t tasks;

public:
    /**
     * Destroy the request by deleting the tasks.
     */
    virtual ~MultiTaskRequest();

    /**
     * Add a task to the request.
     */
    void addTask(Task* task);

    /**
     * Get the number of tasks.
     */
    size_t getNumTasks() const;

    /**
     * Get the task at the given index.
     */
    Task* getTask(size_t index) const;

protected:
    /**
     * Perform the tasks one-by-one.
     */
    virtual void execute();
};

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

/**
 * A templated version of MultiTaskRequest, which accepts and returns
 * tasks of a given type only.
 */
template <class TaskClass>
class MultiTaskRequestTemplate : public MultiTaskRequest
{
public:
    /**
     * Add the given task.
     */
    void addTask(TaskClass* task);

    /**
     * Get the task at the given index.
     */
    TaskClass* getTask(size_t index) const;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline void MultiTaskRequest::addTask(Task* task)
{
    tasks.push_back(task);
}

//------------------------------------------------------------------------------

inline size_t MultiTaskRequest::getNumTasks() const
{
    return tasks.size();
}

//------------------------------------------------------------------------------

inline Task* MultiTaskRequest::getTask(size_t index) const
{
    return tasks[index];
}

//------------------------------------------------------------------------------
// Template definitions
//------------------------------------------------------------------------------

template <class TaskClass>
inline void MultiTaskRequestTemplate<TaskClass>::addTask(TaskClass* task)
{
    MultiTaskRequest::addTask(task);
}

//------------------------------------------------------------------------------

template <class TaskClass>
inline TaskClass*
MultiTaskRequestTemplate<TaskClass>::getTask(size_t index) const
{
    return static_cast<TaskClass*>(MultiTaskRequest::getTask(index));
}

//------------------------------------------------------------------------------

} /* namespace xplra */

//------------------------------------------------------------------------------
#endif // XPLRA_MULTITASKREQUEST_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
